function [A,RES,F]=uncsmo(m1, m2, n1, n2, k1, k2, t1, t2, x1, x2, Y, r1, r2, mu1, mu2)
% function [A,RES,F]=uncsmo(m1, m2, n1, n2, k1, k2, t1, t2, x1, x2, Y, r1, r2,
%                         mu1, mu2)
% unconstrained bivariate smoothing
%
% compute coefficients of a polynomial tensor product spline of order k1 x k2
% in its B-spline representation to given knot sequence t1 x t2 which
% minimizes 
%
%  || [ Y 0 ] -  [           B1(t1) ] * A * [           B2(t2) ]^T ||^2
%  || [ 0 0 ]    [ mu1^0.5 S_r1(t1) ]       [ mu2^0.5 S_r2(t2) ]   ||_F
%
% for the data values Y_{i1,i2} and the abscissae (x1_i1, x2_i2)
% i1=1,...,m1; i2=1,...,m2
%
% Input:               m1;    number of data points in x-direction
%                      m2;    number of data points in y-direction
%                      n1;    number of B-splines in x-direction
%                      n2;    number of B-splines in y-direction
%                      k1;    order of B-Splines in x-direction
%                      k2;    order of B-Splines in y-direction
%            t1[1..n1+k1];    knot sequence in x-direction
%            t2[1..n2+k2];    knot sequence in y-direction
%               x1[1..m1];    data abszissae in x-direction
%               x2[1..m2];    data abszissae in y-direction
%          Y[1..m1,1..m2];    data values
%                      r1;    order of derivative in smoothing term (x-direction)
%                      r2;    order of derivative in smoothing term (y-direction)
%                     mu1;    smoothing parameter (x-direction)
%                     mu2;    smoothing parameter (y-direction)
%
% Output:  A[1..n1,1..n2];    coefficients of spline
%                     RES;    norm of residual matrix
% F[1..m1+n1-r1,1..m2+n2-r2];    residual matrix
  
S1=smooth(t1,n1,k1,r1);
B1=spcol(t1',k1,x1');
S2=smooth(t2,n2,k2,r2);
B2=spcol(t2',k2,x2');
Y0=zeros(m1+n1-r1,m2+n2-r2);
Y0(1:m1,1:m2)=Y;
B10=zeros(m1+n1-r1,n1);
B10=[B1;sqrt(mu1).*S1];
B20=zeros(m2+n2-r2,n2);
B20=[B2;sqrt(mu2).*S2];

A=(B20\((B10\Y0)'))';
F=Y0-B10*A*B20';
RES=norm(F,'fro');
