function [C, h]=relconst(n, k, t, l, pvec, EPSILON)
% function [C, h]=relconst(n, k, t, l, pvec, EPSILON)
% relative constraints
%
% compute matrix C and vector h for linear constraints g(t)=C*t-h >= 0
% which avoid the coalescing of knots for a relative measure of distance
%
% Input:                n;    number of B-splines
%                       k;    order of B-Splines
%               t[1..n+k];    knot sequence
%                       l;    number of free knots
%              pvec[1..l];    index to free knots
%                 EPSILON;    relative measure of distance
%
% Output:  C[1..2*l,1..l];    
%                 h[1..l];    

EPSILON1=1.0-EPSILON;
nrcol=2*l;
C=zeros(nrcol,l);
h=zeros(nrcol,1);

% Generate matrix of constraints  t(i) < t(i+1)
if l==1
  C(1,1)=1;
  h(1)=EPSILON1*t(pvec(1)-1)+EPSILON*t(pvec(1)+1);
  C(2,1)=-1;
  h(2)=-EPSILON1*t(pvec(1)+1)-EPSILON*t(pvec(1)-1);
else
  if (pvec(1)+1==pvec(2))
    C(1,1)=1;
    C(1,2)=-EPSILON;
    h(1)=EPSILON1*t(pvec(1)-1);
    C(2,1)=-1;
    C(2,2)=EPSILON1;
    h(2)=-EPSILON*t(pvec(1)-1);
  else
    C(1,1)=1;
    h(1)=EPSILON1*t(pvec(1)-1)+EPSILON*t(pvec(1)+1);
    C(2,1)=-1;
    h(2)=-EPSILON1*t(pvec(1)+1)-EPSILON*t(pvec(1)-1);
  end

  for j=2:l-1,
    if (pvec(j)-1==pvec(j-1)) & (pvec(j)+1==pvec(j+1)),
      C(2*j-1,j-1)=-EPSILON1;
      C(2*j-1,j)  =1.0;
      C(2*j-1,j+1)=-EPSILON;
      h(2*j-1)    =0.0;
      C(2*j,j-1)  =EPSILON;
      C(2*j,j)    =-1.0;
      C(2*j,j+1)  =EPSILON1;
      h(2*j)      =0.0;
    end,

    if (pvec(j)-1==pvec(j-1)) & (pvec(j)+1~=pvec(j+1)),
      C(2*j-1,j-1)=-EPSILON1;
      C(2*j-1,j)  =1.0;
      h(2*j-1)    =EPSILON*t(pvec(j)+1);
      C(2*j,j-1)  =EPSILON;
      C(2*j,j)    =-1.0;
      h(2*j)      =-EPSILON1*t(pvec(j)+1);
    end,

    if (pvec(j)-1~=pvec(j-1)) & (pvec(j)+1==pvec(j+1)),
      C(2*j-1,j)  =1.0;
      C(2*j-1,j+1)=-EPSILON;
      h(2*j-1)    =EPSILON1*t(pvec(j)-1);
      C(2*j,j)    =-1.0;
      C(2*j,j+1)  =EPSILON1;
      h(2*j)      =-EPSILON*t(pvec(j)-1);
    end,

    if (pvec(j)-1~=pvec(j-1)) & (pvec(j)+1~=pvec(j+1)),
      C(2*j-1,j)  =1.0;
      h(2*j-1)    =EPSILON1*t(pvec(j)-1)+EPSILON*t(pvec(j)+1);
      C(2*j,j)    =-1.0;
      h(2*j)      =-EPSILON1*t(pvec(j)+1)-EPSILON*t(pvec(j)-1);
    end,

  end,

  if (pvec(l)-1==pvec(l-1)),
    C(2*l-1,l-1)=-EPSILON1;
    C(2*l-1,l)  =1.0;
    h(2*l-1)    =EPSILON*t(pvec(l)+1);
    C(2*l,l-1)  =EPSILON;
    C(2*l,l)    =-1.0;
    h(2*l)      =-EPSILON1*t(pvec(l)+1);
  else
    C(2*l-1,l)  =1.0;
    h(2*l-1)    =EPSILON1*t(pvec(l)-1)+EPSILON*t(pvec(l)+1);
    C(2*l,l)    =-1.0;
    h(2*l)      =-EPSILON1*t(pvec(l)+1)-EPSILON*t(pvec(l)-1);
  end,

end % if l==1

