function [alpha,RES,F]=consmo(m, n, k, t, x, y, r, mu, p, L, U)
% function [alpha,RES,F]=consmo(m, n, k, t, x, y, r, mu, p, L, U)
% constrained smoothing
%
% compute coefficients of a polynomial spline of order k in its B-spline
% representation to given knot sequence t which minimizes
%
% || y - B * alpha ||^2 + mu * || S * alpha ||^2 --> Min !
% 
% where || S * alpha ||^2 is an approximation of the smoothing term 
% \int [ s^{r} (x) ]^2 dx
% for the data points x[i], y[i] i=1,...,m
% under the shape constraints L <= alpha^p <= U
%
% Input:                m;    number of data points
%                       n;    number of B-splines
%                       k;    order of B-Splines
%               t[1..n+k];    knot sequence
%                 x[1..m];    data abszissae
%                 y[1..m];    data values
%                       r;    order of derivative in smoothing term
%                      mu;    smoothing parameter
%                       p;    order of derivative in constraints
%               L[1..n-p];    lower bounds on alpha^p
%               U[1..n-p];    upper bounds on alpha^p
%
% Output:     alpha[1..n];    coefficients of spline
%                     RES;    norm of residual vector
%             F[1..m+n-r];    residual vector

S=smooth(t,n,k,r);
B=spcol(t',k,x');
D=deriv(t,n,k,p);
y0=zeros(m+n-r,1);
y0(1:m)=y;
B0=zeros(m+n-r,n);
B0=[B;sqrt(mu).*S];
A=[-D;D];
b=[-L;U];

OPTIONS=optimset('LargeScale','off','Display','off');
[alpha,tRES,tF]=lsqlin(B0,y0,A,b,[],[],[],[],[],OPTIONS);
RES=sqrt(tRES);
F=-tF;

