function [alpha,RES,F]=consmols(m, n, k, t, x, y, r, mu, p, L, U)
% function [alpha,RES,F]=consmols(m, n, k, t, x, y, r, mu, p, L, U)
% constrained smoothing with LSSOL package
%
% compute coefficients of a polynomial spline of order k in its B-spline
% representation to given knot sequence t which minimizes
%
% || y - B * alpha ||^2 + mu * || S * alpha ||^2 --> Min !
% 
% where || S * alpha ||^2 is an approximation of the smoothing term 
% \int [ s^{r} (x) ]^2 dx
% for the data points x[i], y[i] i=1,...,m
% under the shape constraints L <= alpha^p <= U
%
% Input:                m;    number of data points
%                       n;    number of B-splines
%                       k;    order of B-Splines
%               t[1..n+k];    knot sequence
%                 x[1..m];    data abszissae
%                 y[1..m];    data values
%                       r;    order of derivative in smoothing term
%                      mu;    smoothing parameter
%                       p;    order of derivative in constraints
%               L[1..n-p];    lower bounds on alpha^p
%               U[1..n-p];    upper bounds on alpha^p
%
%                             On output y will be overwritten! If you
%                             don't want this, you should call
%
%                             ysave=y+0;
%                             [alpha,RES]=consmols(m,n,k,t,x,y,r,mu,p,L,U);
%                             y=ysave;   
%
%                             to avoid a bug in Matlab!
%
% Output:     alpha[1..n];    coefficients of spline
%                     RES;    norm of residual vector
%             F[1..m+n-r];    residual vector

S=smooth(t,n,k,r);
B=spcol(t',k,x');
D=deriv(t,n,k,p);
y0=zeros(m+n-r,1);
y0(1:m)=y;
B0=zeros(m+n-r,n);
B0=[B;sqrt(mu).*S];

c=zeros(n,1);
l=-inf.*ones(n+n-p,1);
u= inf.*ones(n+n-p,1);
l(n+1:2*n-p)=L;
u(n+1:2*n-p)=U;
alpha0=zeros(n,1);
F=y0;
[alpha,obj]=lssol(B0,y0,c,D,alpha0,l,u,'ls1',0);
RES=sqrt(2*obj);
F=F-B0*alpha;
