function [alpha,RES,F]=uncsmo(m, n, k, t, x, y, r, mu)
% function [alpha,RES,F]=uncsmo(m, n, k, t, x, y, r, mu)
% unconstrained smoothing
%
% compute coefficients of a polynomial spline of order k in its B-spline
% representation to given knot sequence t which minimizes
%
% || y - B * alpha ||^2 + mu * || S * alpha ||^2 --> Min !
% 
% where || S * alpha ||^2 is an approximation of the smoothing term 
% \int [ s^{r} (x) ]^2 dx
% for the data points x[i], y[i] i=1,...,m
%
% Input:                m;    number of data points
%                       n;    number of B-splines
%                       k;    order of B-Splines
%               t[1..n+k];    knot sequence
%                 x[1..m];    data abszissae
%                 y[1..m];    data values
%                       r;    order of derivative in smoothing term
%                      mu;    smoothing parameter
%
% Output:     alpha[1..n];    coefficients of spline
%                     RES;    norm of residual vector
%             F[1..m+n-r];    residual vector

S=smooth(t,n,k,r);
B=spcol(t',k,x');
y0=zeros(m+n-r,1);
y0(1:m)=y;
B0=zeros(m+n-r,n);
B0=[B;sqrt(mu).*S];

alpha=B0\y0;
F=y0-B0*alpha;
RES=norm(F,2);